function mappingEndpointFormatter(value, row, index) {
	var options = endpointOptions.slice()
	options.unshift(['<option value="">'+_('Use device emergency caller id')+'</option>'])
	var html = $(`<select class="form-control endpointMap emergencyMap" data-id="${row.id}">${options.join("\n")}</select>`);
	if(value) {
		html.find(`option[value=${value}]`).attr('selected', 'selected');
	}

	return html[0].outerHTML;
}

function mappingLocationFormatter(value, row, index) {
	var options = locationOptions.slice()
	options.unshift(['<option value="">'+_('Use device emergency caller id')+'</option>'])
	var html = $(`<select class="form-control locationMap emergencyMap" data-id="${row.id}">${options.join("\n")}</select>`);
	html.prop('disabled', !row.endpoint)
	if(value) {
		html.find(`option[value=${value}]`).attr('selected', 'selected');
	}
	return html[0].outerHTML;
}

function didLinkFormatter(value, row, index){
	var html = '';
	html += `<a href=# class="editlink" data-id="${row.entity_id}" data-name="${row.name}" data-description="${row.description}"><i class="fa fa-edit"></i></a>`;
	return html;
}

function smsLinkFormatter(value, row, index){
	var html = '';
	html += `<a href=# class="editlink" data-id="${row.stripped_e164}" data-name="${row.name}" data-users="${JSON.stringify(row.sms_user_map)}"><i class="fa fa-edit"></i></a>`;
	return html;
}

function locationsMapFormatter(value, row, index){
	return locationsMap[value] || value;
}

function routeFailoverFormatter(value, row, index) {
	var description = '';
	if(row.failover.type === 'NONE') {
		description = 'Not Configured';
	} else {
		description = row.failover.type +": "+row.failover.uri
	}

	if (row.failover.type === 'LOCATION') {
		return row.failover.type +": "+row.failover.uri.replace('location://', '');
	}
	return `<a href="#"  class="failoverlink" data-id="${row.entity_id}" data-name="${row.name}" data-type="${row.failover.type}" data-uri="${row.failover.uri}" data-when="${row.failover.when}" data-timeout="${row.failover.timeout}"><i class="fa fa-edit"></i>${description}</a>`;
}

function routeDestinationFormatter(value, row, index) {
	if(!row.route) {
		return 'Missing';
	}

	var destination = row.route.destination;
	var description = '';

	if (destinations[destination]) {
		description = destinations[destination].description;
	}

	return `<a href="#"  class="destlink" data-id="${row.entity_id}" data-name="${row.name}" data-dest="${destination}" data-extension="${row.inbound}"><i class="fa fa-edit"></i>${description}</a>`;
}

function e911LocationAddressFormatter(value, row, index){
	return value.replace(/\n/g,'<br />');
}

function e911LocationLinkFormatter(value, row, index){
	var html = '';
	html += `<a href=# class="editlink" data-id="${row.entity_id}" data-name="${row.name}" data-description="${row.description}"><i class="fa fa-edit"></i></a>`;
	return html;
}
function e911EndpointLinkFormatter(value, row, index){
	var html = '';
	html += `<a href=# class="editlink" data-id="${row.entity_id}" data-name="${row.name}" data-description="${row.description}" data-caller-name="${row.caller_name}" data-call-back-number="${row.call_back_number}" data-pbx-caller-id="${row.pbx_caller_id}" data-default-location="${row.default_location}"><i class="fa fa-edit"></i></a>`;
	return html;
}
function e911MappingLinkFormatter(value, row, index){
	var html = '';
	html += `<a href=# class="editlink" data-id="${row.id}" data-endpoints='${JSON.stringify(row.mappingPermissions.endpoints)}' data-locations='${JSON.stringify(row.mappingPermissions.locations)}'><i class="fa fa-edit"></i></a>`;
	return html;
}
function e911NotificationLinkFormatter(value, row, index){
	var html = '';
	html += `<a href="#" id="notification-${row.route_id}" data-name="${row.name}" class="edit-notification-link" data-id="${row.route_id}" data-notifications='${JSON.stringify(row.notifications)}'><i class="fa fa-edit"></i></a>`;
	return html;
}

function manageNotificationsLinkFormatter(value, row, index){
	var html = '';
	html += `<a href="?display=clearlytrunking&action=editnotification&id=${row.id}"><i class="fa fa-edit" id="notification-${row.entity_id}"></i></a>`;
	return html;
}
function e911NotificationEnabledFormatter(value, row, index){
	var html = '';
	var oclass = '';
	if(typeof(row['notifications']['sms']) !== 'undefined'){
		oclass = row['notifications']['sms']?'text-success':'text-danger';
	}else{
		oclass = 'text-danger';
	}
	html += `<i class="fa fa-mobile ${oclass}" aria-hidden="true"></i>&nbsp;`;
		if(typeof(row['notifications']['pagegroup']) !== 'undefined'){
		oclass = row['notifications']['pagegroup']?'text-success':'text-danger';
	}else{
		oclass = 'text-danger';
	}
	html += `<i class="fa fa-users ${oclass}" aria-hidden="true"></i>&nbsp;`;
		if(typeof(row['notifications']['email']) !== 'undefined'){
		oclass = row['notifications']['email']?'text-success':'text-danger';
	}else{
		oclass = 'text-danger';
	}
	html += `<i class="fa fa-envelope-open-o ${oclass}" aria-hidden="true"></i>&nbsp;`;
	return html;
}

$("#refresh").click(function() {
	$(this).prop('disabled', true);
	$.post( "ajax.php?module=clearlytrunking&command=clearCache" )
	.done(function() {
		location.reload();
	})
	.fail(function() {
		$("#refresh").prop('disabled', false);
	})
})

$("#remove-key").click(function() {
	if(confirm(_("Are you sure you want to delete the key?"))) {
		$(this).prop('disabled', true);
		$.post( "ajax.php?module=clearlytrunking&command=removeKey" )
			.done(function() {
				window.location = 'config.php?display=clearlytrunking&reload='+Date.now();
			})
			.fail(function() {
				$("#remove-key").prop('disabled', false);
			})
	}
})

function updateE911Mapping(device, endpoint, location) {
	$(`.emergencyMap[data-id=${device}]`).prop('disabled', true);
	$.post( "ajax.php?module=clearlytrunking&command=editE911Mapping", {id: device, endpoint: endpoint, location: location} )
		.always(function() {
			$(`.emergencyMap[data-id=${device}]`).prop('disabled', false);
		})
}

$("#emergency-mapping").on("post-body.bs.table", function (evt, data) {
	$(this).find(".endpointMap").change(function() {
		var device = $(this).data('id');
		$(`.locationMap[data-id=${device}]`).prop('disabled', !$(this).val());
		if(!$(this).val()) {
			$(`.locationMap[data-id=${device}]`).val("");
		}
		updateE911Mapping(device, $(this).val(), $(`.locationMap[data-id=${device}]`).val())
	})
	$(this).find(".locationMap").change(function() {
		var device = $(this).data('id');
		updateE911Mapping(device, $(`.endpointMap[data-id=${device}]`).val(), $(this).val())
	})
	$(this).find(".editlink").click(function() {
		var id = $(this).data("id");
		var endpoints = $(this).data("endpoints");
		var locations =  $(this).data("locations");
		$("#emergency-mapping-endpoint").html(endpointOptions).multiselect('rebuild').multiselect('select',endpoints);
		$("#emergency-mapping-location").html(locationOptions).multiselect('rebuild').multiselect('select',locations);
		$('#edit-emergency-mapping .modal-title').text('Edit User Emergency Change Control for Device ' + id);
		$("#edit-emergency-mapping").data('id',id);
		$('#edit-emergency-mapping').modal('show')
	});
});

$("#edit-emergency-mapping-save").click(function() {
	var id = $("#edit-emergency-mapping").data("id");
	var endpoints = $("#emergency-mapping-endpoint").val();
	var locations = $("#emergency-mapping-location").val();
	$('#edit-emergency-mapping').modal('hide')

	$(this).prop('disabled', true);
	$.post( "ajax.php?module=clearlytrunking&command=configureEmergencyMappingSettings", {id: id, endpoints: endpoints, locations: locations} )
	.done(function(res) {
		$("#emergency-mapping").bootstrapTable('refresh');
		if(res.reload) {
			toggle_reload_button("show");
		}
		$('#edit-emergency-mapping').modal('hide');

	})
	.always(function() {
		$("#edit-emergency-mapping-save").prop('disabled', false);
	});
})

$("#sms-number-list").on("post-body.bs.table", function (evt, data) {
	$(this).find(".editlink").click(function() {
		var id = $(this).data("id");
		var name = $(this).data("name");
		var users = $(this).data("users");
		$("#edit-number-sms").data('id',id);
		$('#number-sms-assigned').multiselect('deselectAll', false).multiselect('select', users);
		$('#edit-number-sms .modal-title').text('Edit SMS Routing for '+name)
		$('#edit-number-sms').modal('show')
	});
});

$("#edit-number-sms-save").click(function() {
	var id = $("#edit-number-sms").data("id");
	var users = $('#number-sms-assigned').val();
	$(this).prop('disabled', true);
	$.post( "ajax.php?module=clearlytrunking&command=configureSmsUsers", {id: id, users: users} )
	.done(function(res) {
		$("#sms-number-list").bootstrapTable('refresh');
		if(res.reload) {
			toggle_reload_button("show");
		}
		$('#edit-number-sms').modal('hide');

	})
	.always(function() {
		$("#edit-number-sms-save").prop('disabled', false);
	});
});

$("#e911notification").on("post-body.bs.table", function (evt, data) {
	$(this).find(".edit-notification-link").click(function() {
		var id = $(this).data("id");
		var notif = $(this).data("notifications");
		const {sms, email, pagegroup, ...selections} = notif;
		var name = $(this).data("name");
		$('#routenotifysetting').data('id',id);
		$('#routenotifysetting').multiselect('deselectAll', false);
		$('#routenotifysetting').multiselect('select', Object.values(selections));
		$('#edit-route-notification').modal('show');
		$('#edit-route-notification .modal-title').text('Edit notifications for: '+name);
	});
});

	$("#endpoint-list").on("post-body.bs.table", function (evt, data) {
	if(data.length) {
		var defaultEndpoint = $("#location-default-endpoint").data('id');
		endpointOptions = [];

		$.each(data, function(k,v) {
			endpointsMap[v.entity_id] = v.name
			endpointOptions.push(`<option value="${v.entity_id}">${v.name}</option>`)
			if(defaultEndpoint === v.entity_id) {
				$("#location-default-endpoint").text(v.name)
			}
		})
	}

	$(this).find(".editlink").click(function() {
		var id = $(this).data("id");
		var name = $(this).data("name");
		var description = $(this).data("description");
		var callerName = $(this).data("caller-name");
		var callBackNumber = $(this).data("call-back-number");
		var callerId = $(this).data("pbx-caller-id");
		var defaultLocation = $(this).data("default-location");
		$('#edit-endpoint').data('id',id);
		$('#edit-endpoint').modal('show')
		$('#edit-endpoint .modal-title').text('Edit '+name)
		$('#endpoint-name').val(name)
		$('#endpoint-description').val(description)
		$('#endpoint-caller-name').val(callerName)
		$('#endpoint-call-back-number').val(callBackNumber)
		$('#endpoint-pbx-caller-id').val(callerId)
		$('#endpoint-default-location').val(defaultLocation)
	});
});

$("#location-list").on("post-body.bs.table", function (data) {
	if(data.length) {
		locationOptions = [];
		$.each($(this).bootstrapTable('getData'), function(k,v) {
			locationsMap[v.entity_id] = v.name
			locationOptions.push(`<option value="${v.entity_id}">${v.name}</option>`)
		})
	}
	$("#endpoint-default-location").html(locationOptions);
	$(this).find(".editlink").click(function() {
		var id = $(this).data("id");
		var name = $(this).data("name");
		var description = $(this).data("description");
		$('#edit-location').data('id',id);
		$('#edit-location').modal('show')
		$('#edit-location .modal-title').text('Edit '+name)
		$('#location-name').val(name)
		$('#location-description').val(description)
	});
});

$("#number-list").on("post-body.bs.table", function () {
	$(this).find(".editlink").click(function() {
		var id = $(this).data("id");
		var name = $(this).data("name");
		var description = $(this).data("description");
		$('#edit-number').data('id',id);
		$('#edit-number').modal('show')
		$('#edit-number .modal-title').text('Edit '+name)
		$('#number-name').val(name)
		$('#number-description').val(description)
	});

	$(this).find(".failoverlink").click(function() {
		var id = $(this).data("id");
		var name = $(this).data("name");
		var type = $(this).data("type");
		if(type === 'NONE') {
			$(".number-failover-additional").hide();
		} else {
			$(".number-failover-additional").show();
		}
		var uri = $(this).data("uri");
		var timeout = $(this).data("timeout");
		var when = $(this).data("when");
		$('#edit-number-failover').data('id',id);
		$('#edit-number-failover').modal('show')
		$('#edit-number-failover .modal-title').text('Edit Failover for: '+name)
		$('#number-type').val(type)
		$('#number-uri').val(uri)
		$('#number-timeout').val(timeout || 5)
		$('#number-when').val(when || 'UNREACHABLE')
	});

	$(this).find(".destlink").click(function() {
		resetDrawselects();
		var extension = $(this).data("extension");
		var name = $(this).data("name");
		var dest = $(this).data("dest");
		if (typeof destinations[dest] !== undefined) {
			if (destinations[dest].category === undefined) {
				//make sure we try to give it a category
				destinations[dest].category = destinations[dest].name;
			}
		}
		setDrawselect('goto0', dest);
		$('#edit-destination').data('extension',extension);
		$('#edit-destination').modal('show')
		$('#edit-destination .modal-title').text('Edit Desintation for '+name)
	});
});

$("#save-notifications").click(function(){
	var selections = [];
	var id = $('#routenotifysetting').data('id');
	$('#routenotifysetting').children("option:selected").each(
		function(){
			selections.push($(this).val());
		}
	);
	if(!Array.isArray(selections)){
		selections = [selections];
	}
	$.post( "ajax.php?module=clearlytrunking&command=updatenotifications", {id: id, selections: selections})
	.done(function(ret){
		if(!ret.status){
			fpbxToast(ret.message);
		}
		if(ret.status){
			$("#e911notification").bootstrapTable('refresh');
			toggle_reload_button("show");
			$("#edit-route-notification").modal('hide');
		}
	});
});

$("#edit-number-save").click(function() {
	$(this).prop('disabled', true);
	var id = $('#edit-number').data('id');
	$.post( "ajax.php?module=clearlytrunking&command=editNumber", {id: id, name: $("#number-name").val(), description: $("#number-description").val()} )
		.done(function() {
			$("#number-list").bootstrapTable('refresh');
			if($("#sms-number-list").length) {
				$("#sms-number-list").bootstrapTable('refresh');
			}
			$('#edit-number').modal('hide')
		})
		.always(function() {
			$("#edit-number-save").prop('disabled', false);
		})
})

$("#configure-outbound-routes-save").click(function() {
	$(this).prop('disabled', true);
	$.post( "ajax.php?module=clearlytrunking&command=configureOutboundRoutes", {cid: $("#location-cid").val()} )
	.done(function(res) {
		fpbxToast('Successfully configured routes');
		if(res.reload) {
			toggle_reload_button("show");
		}
		$("#configure-outbound-routes-modal").modal('hide');

	})
	.always(function() {
		$("#configure-outbound-routes-save").prop('disabled', false);
	})
});

$("#configure-outbound-routes").click(function() {
	$("#configure-outbound-routes-modal").modal('show');
});

$("#configure-inbound-routes").click(function() {
	$(this).prop('disabled', true);
	$.post( "ajax.php?module=clearlytrunking&command=configureInboundRoutes" )
	.done(function(res) {
		fpbxToast('Successfully configured routes');
		$("#number-list").bootstrapTable('refresh');
		if(res.reload) {
			toggle_reload_button("show");
		}
	})
	.always(function() {
		$("#configure-inbound-routes").prop('disabled', false);
	})
});

$("#edit-route-save").click(function() {
	var extension = $('#edit-destination').data('extension');
	var gotoc = $('#goto0').val();
	var gotoid = $('#goto0').data('id');
	var destination = $("#" + gotoc + gotoid).val();

	$(this).prop('disabled', true);
	$.post( "ajax.php?module=clearlytrunking&command=configureInboundRouteDestination", {extension: extension, destination: destination} )
	.done(function(res) {
		$("#number-list").bootstrapTable('refresh');
		toggle_reload_button("show");
		$("#edit-destination").modal('hide');
	})
	.always(function() {
		$("#edit-route-save").prop('disabled', false);
	})
});

$("#edit-number-failover-save").click(function() {
	var id = $('#edit-number-failover').data('id');
	var type = $("#number-type").val()
	var uri = $('#number-uri').val()
	var timeout = $('#number-timeout').val()
	var when = $('#number-when').val()

	$(this).prop('disabled', true);
	$.post( "ajax.php?module=clearlytrunking&command=editNumberFailover", {id: id, type: type, uri: uri, timeout: timeout, when: when} )
	.done(function(res) {
		$("#number-list").bootstrapTable('refresh');
		$("#edit-number-failover").modal('hide');
	})
	.always(function() {
		$("#edit-number-failover-save").prop('disabled', false);
	})
});

$('#number-type').change(function() {
	if($(this).val() === "NONE") {
		$(".number-failover-additional").hide();
	} else {
		$(".number-failover-additional").show();
	}
});

$('#type').change(function() {
	if($(this).val() === "NONE") {
		$(".failover-additional").hide();
	} else {
		$(".failover-additional").show();
	}
});

if($('#type').val() === 'NONE') {
	$(".failover-additional").hide();
} else {
	$(".failover-additional").show();
}

$("#edit-failover").click(function() {
	$('#edit-failover-modal').modal('show')
});

$("#edit-failover-save").click(function() {
	var type = $("#type").val()
	var uri = $('#uri').val()
	var timeout = $('#timeout').val()
	var when = $('#when').val()

	$(this).prop('disabled', true);
	$.post( "ajax.php?module=clearlytrunking&command=editFailover", {type: type, uri: uri, timeout: timeout, when: when} )
	.done(function(res) {
		location.reload();
	})
	.always(function() {
		$("#edit-failover-save").prop('disabled', false);
	})
});

$("#edit-endpoint-save").click(function() {
	var id = $('#edit-endpoint').data('id');
	var name = $("#endpoint-name").val()
	var description = $('#endpoint-description').val()
	var callerName = $('#endpoint-caller-name').val()
	var callBackNumber = $('#endpoint-call-back-number').val()
	var pbxCallerId = $('#endpoint-pbx-caller-id').val()
	var defaultLocation = $('#endpoint-default-location').val()

	$(this).prop('disabled', true);
	$.post( "ajax.php?module=clearlytrunking&command=editEndpoint", {id: id, name: name, description: description, callerName: callerName, callBackNumber: callBackNumber, pbxCallerId: pbxCallerId, defaultLocation: defaultLocation} )
	.done(function(res) {
		$("#endpoint-list").bootstrapTable('refresh');
		$("#edit-endpoint").modal('hide');
	})
	.always(function() {
		$("#edit-endpoint-save").prop('disabled', false);
	})
});

$("#edit-location-save").click(function() {
	var id = $('#edit-location').data('id');
	var name = $("#location-name").val()
	var description = $('#location-description').val()

	$(this).prop('disabled', true);
	$.post( "ajax.php?module=clearlytrunking&command=editLocation", {id: id, name: name, description: description} )
	.done(function(res) {
		$("#location-list").bootstrapTable('refresh');
		$("#edit-location").modal('hide');
	})
	.always(function() {
		$("#edit-location-save").prop('disabled', false);
	})
});
$(document).on('ready', function(){
	$('#routenotifysetting').multiselect();
	let searchParams = new URLSearchParams(window.location.search);
	if(searchParams.has('tab')){
		let tab = searchParams.get('tab');
		switch(tab){
			case 'editnotify':
				$('[href="#e911settings"]').tab('show');
				$('[href="#managenotifications"]').tab('show');
				toggle_reload_button("show");
			break;
		}
	}

	$('#daemon-start').on('click', function(e){
		e.preventDefault();
		$("#daemon-start daemon-stop daemon-restart").attr('disabled', true);
		$.get(`${FreePBX.ajaxurl}?module=clearlytrunking&command=startdaemon`, function(data) {
			$("#daemon-start daemon-stop daemon-restart").attr('disabled', false);
			fpbxToast('Daemon Starting, This may take a moment');
		});
	});

	$('#daemon-stop').on('click', function(e){
		e.preventDefault();
		$("#daemon-start daemon-stop daemon-restart").attr('disabled', true);
		$.get(`${FreePBX.ajaxurl}?module=clearlytrunking&command=stopdaemon`, function(data) {
			$("#daemon-start daemon-stop daemon-restart").attr('disabled', false);
			fpbxToast('Daemon Stopping, This may take a moment');
		});
	});

	$('#daemon-restart').on('click', function(e){
		e.preventDefault();
		$("#daemon-start daemon-stop daemon-restart").attr('disabled', true);
		$.get(`${FreePBX.ajaxurl}?module=clearlytrunking&command=restartdaemon`, function(data) {
			$("#daemon-start daemon-stop daemon-restart").attr('disabled', false);
			fpbxToast('Daemon Restarting, This may take a moment');
		});
	});

	/** Status Poll */
	function doPoll(){
		var CurrentDate = moment().unix();
		$.get(`${FreePBX.ajaxurl}?module=clearlytrunking&command=daemonstatus`, function(data) {
			if(data.status){
				$('#daemon-status').addClass('label-success');
				$('#daemon-status').removeClass('label-default');
				$('#daemon-status').removeClass('label-danger');
				$("#daemon-refresh-time").html(moment().format(`${dateformat} - ${timeformat}`));
			}else{
				$('#daemon-status').removeClass('label-success');
				$('#daemon-status').removeClass('label-default');
				$('#daemon-status').addClass('label-danger');
				$("#daemon-refresh-time").html(moment().format(`${dateformat} - ${timeformat}`));
			}
			$("#daemon-status").html(`${data.state}`);
			setTimeout(doPoll,30000);
		});
	}
	doPoll();
});
